%% data calibration
function [temp_value,batt_value,lux_value] = convert(temp_code, batt_code, lux_code, ...
        batt_value_1pt, batt_code_1pt, temp_code_25c, temp_code_45c, temp_code_55c, ...
        fc_batt, lux_shift, vbat_fixed)
    
    if vbat_fixed == 0
        if batt_code == batt_code_1pt
            batt_value = batt_value_1pt;
        else
            batt_value = (-1/16)*(batt_code - batt_code_1pt) + batt_value_1pt;
        end
    else
       batt_value = vbat_fixed;
    end
    
    if temp_code < temp_code_45c
        temp_value = (20 / (temp_code_45c - temp_code_25c)) * (temp_code - temp_code_25c) + 25;
    else
        temp_value = (10 / (temp_code_55c - temp_code_45c)) * (temp_code - temp_code_45c) + 45;
    end 
          
    temp_range = [25 35 45 55];
    batt_range = [3.9 4.0 4.1 4.2];
    lux_range = [500 1000 5000 10000 50000 100000];

    temp_len = length(temp_range);
    batt_len = length(batt_range);
    lux_len = length(lux_range);
    
    for ii = 1:lux_len
        fc_temp{ii} = fit(temp_range',[fc_batt{ii,1}(batt_value),fc_batt{ii,2}(batt_value),fc_batt{ii,3}(batt_value),fc_batt{ii,4}(batt_value)]','linearinterp');
    end
    
    fc_lux = fit([fc_temp{1}(temp_value),fc_temp{2}(temp_value),fc_temp{3}(temp_value),fc_temp{4}(temp_value),fc_temp{5}(temp_value),fc_temp{6}(temp_value)]',log10(lux_range)','linearinterp');
     
    lux_value = 10^(fc_lux(log10(lux_code) + lux_shift));
     
    param_a = 0.79;
    param_b = 90000/(221483-5000);
    param_c = 20000;
        
    lux_value = lux_value * param_a;  
    
    if lux_value < 1
    	lux_value = 1;
    elseif lux_value > param_c
        lux_value = param_c + (lux_value - param_c) * param_b;
    end 

end


