clear all; close all;

% Calibrate data from snail stacks using meausured temperature & battery voltage and calibration data
% This is only for the sensor deployed on 8/5/2017 and refound on 8/8/2017

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Please input the request information %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
file_name = '20170812_S05';
stack_name = 'S05';

measurement_time_start = datenum('2017/08/08 05:10 AM');
measurement_time_end = measurement_time_start + (1/24/6)*250;

request_time_start = '2017/08/09 06:00 AM';
request_time_end = '2017/08/09 07:30 PM';

%%%%%%%%%%%%%%%%%%%%%%
% Program Start Here %
%%%%%%%%%%%%%%%%%%%%%%
fid = fopen(strcat(sprintf('%s.csv',file_name)));
d = textscan(fid,'%s','delimiter',',');
d = d{1,1};
rx_data_hex = dec2hex(bin2dec(d));
fclose('all');

cnt = 0;
temp_code = zeros(1,floor(length(rx_data_hex/4)));
batt_code = zeros(1,floor(length(rx_data_hex/4)));
ldoon_code = zeros(1,floor(length(rx_data_hex/4)));
lux_code = zeros(1,floor(length(rx_data_hex/4)));

for i = 1:length(rx_data_hex)
    
    header = rx_data_hex(i,1:3);
    
    if (cnt == 0) && (sum(header == 'A00') ~= 3)

    elseif sum(header == 'A00') == 3
        cnt = cnt + 1;
        temp_code(cnt) = hex2dec(rx_data_hex(i,4:6));
    elseif sum(header == 'B00') == 3
        ldoon_code(cnt) = hex2dec(rx_data_hex(i,4));
        batt_code(cnt) = hex2dec(rx_data_hex(i,5:6));
    elseif sum(header == 'C00') == 3
        lux_data_msb = hex2dec(rx_data_hex(i,4:6));
    elseif sum(header == 'D00') == 3
        lux_code(cnt) = lux_data_msb * 2^12 + hex2dec(rx_data_hex(i,4:6));
    end
    
end

temp_code = temp_code(cnt:-1:1);
ldoon_code = ldoon_code(cnt:-1:1);
batt_code = batt_code(cnt:-1:1);
lux_code = lux_code(cnt:-1:1)*0.1;


%%%%%%%%%%%%%%%%%%
% Get calib data %
%%%%%%%%%%%%%%%%%%
[stack_id, batt_value_1pt, batt_code_1pt, temp_code_25c_5klux, temp_code_45c_5klux, temp_code_55c_5klux, lux_code_25c_5klux] = ...
    textread('../calib_data.txt',...
    '%s %f %f %f %f %f %f','headerlines',3);
stack_index = find(strcmp(stack_id,stack_name));

batt_value_1pt = batt_value_1pt(stack_index);
batt_code_1pt = batt_code_1pt(stack_index);
temp_code_25c_5klux = temp_code_25c_5klux(stack_index);
temp_code_45c_5klux = temp_code_45c_5klux(stack_index);
temp_code_55c_5klux = temp_code_55c_5klux(stack_index);
lux_code_25c_5klux = lux_code_25c_5klux(stack_index);

%%%%%%%%%%%%%%%%%%
% Get model data %
%%%%%%%%%%%%%%%%%%
[lux_model] = textread('../snail_model.txt','%f');

time_range = linspace(measurement_time_start,measurement_time_end,cnt);  
temp_range = [25 35 45 55];
batt_range = [3.9 4.0 4.1 4.2];
lux_range = [500 1000 5000 10000 50000 100000];

temp_len = length(temp_range);
batt_len = length(batt_range);
lux_len = length(lux_range);

for ii = 1:lux_len
    for jj = 1:temp_len
        start = ii+24*(jj-1);
        lux_model_set = [lux_model(start:6:start+18)];
        fc_batt{ii,jj} = fit(batt_range',lux_model_set,'linearinterp');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%
% Convert code to value %
%%%%%%%%%%%%%%%%%%%%%%%%%

lux_shift = lux_model(21) - log10(lux_code_25c_5klux); %% 21 -> 25C, 4.2V, 5klux

temp_value = zeros(1,cnt);
batt_value = zeros(1,cnt);
lux_value = zeros(1,cnt);   
vbat_fixed = 0;

for kk = 1:cnt
    [temp_value(kk),batt_value(kk),lux_value(kk)] = convert(temp_code(kk), batt_code(kk), lux_code(kk), ...
        batt_value_1pt, batt_code_1pt, temp_code_25c_5klux, temp_code_45c_5klux, temp_code_55c_5klux, ...
        fc_batt, lux_shift, vbat_fixed);
end

if round(max(batt_value)*10) >= 45
    vbat_fixed = (batt_value(1) + batt_value(length(batt_value))) / 2;
    for kk = 1:cnt
        [temp_value(kk),batt_value(kk),lux_value(kk)] = convert(temp_code(kk), batt_code(kk), lux_code(kk), ...
            batt_value_1pt, batt_code_1pt, temp_code_25c_5klux, temp_code_45c_5klux, temp_code_55c_5klux, ...
            fc_batt, lux_shift, vbat_fixed);
    end
end
   
%%%%%%%%%%%%%%%%%%%
% Generate Output %
%%%%%%%%%%%%%%%%%%%
request_time_step = 1/24/6;
request_time_stamp = [datenum(request_time_start):request_time_step:datenum(request_time_end)];
output_lux = zeros(length(request_time_stamp));

time_start = datenum(request_time_start);
time_end = datenum(request_time_end);
time_stamp = [time_start:request_time_step:time_end]; 

output_lux = round(interp1(time_range,lux_value,time_stamp));

Time = time_stamp';
Output_Lux = output_lux';

result_table = table(Time,Output_Lux);
writetable(result_table,sprintf('%s_out.csv',file_name));

%%%%%%%%%%%%%
% Debugging %
%%%%%%%%%%%%%
Time = time_range';
Output_Lux = lux_value';
result_table = table(Time,Output_Lux);
writetable(result_table,sprintf('%s_out_2days.csv',file_name));

